<?php
/*
Plugin Name: Gcal Sidebar
Plugin URI: http://www.oriontechnologysolutions.com/gcal-sidebar/
Description: Pulls a Google Calendar feed and displays it in your sidebar.
Version: 0.2
Author: Orion Technology Solutions
Author URI: http://www.oriontechnologysolutions.com
*/

/*  Copyright 2010  Orion Technology Solutions  (email : wordpress@oriontechnologysolutions.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Thank you Justin Bennett fr providing the initial foundation this was built from.
*/
class GcalSidebar extends WP_Widget {
    /** constructor */
    function GcalSidebar() {
        parent::WP_Widget(false, $name = 'GcalSidebar');	
    }

    /** @see WP_Widget::widget */
    function widget($args, $instance) {		
        extract($args);
        $feed_id = $instance['feed_id'];
        $feed_title = $instance['title'];
        $title_url_option = get_option('title_url_option');
        $title = $this->get_title($feed_title, $feed_id, $title_url_option);

        echo $before_widget;
        echo $before_title . $title . $after_title;
        $this->display_feed($feed_id, $instance);

        echo $after_widget;
    }

    /** @see WP_Widget::update */
    function update($new_instance, $old_instance) {				
        return $new_instance;
    }

    /** @see WP_Widget::form */
    function form($instance) {				
        $feed_id = esc_attr($instance['feed_id']);
        $title = esc_attr($instance['title']);
        $max_results = esc_attr($instance['max_results']);
        $tz_offset = esc_attr($instance['tz_offset']);
        $static_url_option = esc_attr($instance['static_url_option']);
        $static_url = esc_attr($instance['static_url']);
        $title_url_option = esc_attr($instance['static_url']);
        $title_url = esc_attr($instance['static_url']);

        ?>
        <p><label for="<?php echo $this->get_field_id('feed_id'); ?>"><?php _e('Feed ID:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('feed_id'); ?>" name="<?php echo $this->get_field_name('feed_id'); ?>" type="text" value="<?php echo $feed_id; ?>" /></label></p>

        <p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Calendar Title:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label></p>

        <p><label for="<?php echo $this->get_field_id('title_url_option'); ?>"><?php _e('Title URL:'); ?>
            <select name="<?php echo $this->get_field_id('title_url_option'); ?>" id="<?php echo $this->get_field_id('title_url_option'); ?>" >
                <option value="0" <?php echo ($title_url_option == 0) ? 'selected="selected"' : ''; ?>>None</option>
                <option value="1" <?php echo ($title_url_option == 1) ? 'selected="selected"' : ''; ?>>iCal</option>
                <option value="2" <?php echo ($title_url_option == 2) ? 'selected="selected"' : ''; ?>>HTML</option>
                <option value="3" <?php echo ($title_url_option == 3) ? 'selected="selected"' : ''; ?>>Specified</option>
            </select>
            <input class="widefat" id="<?php echo $this->get_field_id('title_url'); ?>" name="<?php echo $this->get_field_name('title_url'); ?>" type="text" value="<?php echo $static_url; ?>" /></label>
        </p>

        <p><label for="<?php echo $this->get_field_id('static_url_option'); ?>"><?php _e('Single Event URL:'); ?>
            <select name="<?php echo $this->get_field_id('static_url_option'); ?>" id="<?php echo $this->get_field_id('static_url_option'); ?>" >
                <option value="0" <?php echo ($static_url_option == 0) ? 'selected="selected"' : ''; ?>>No</option>
                <option value="1" <?php echo ($static_url_option == 1) ? 'selected="selected"' : ''; ?>>Yes</option>
            </select>
            <input class="widefat" id="<?php echo $this->get_field_id('static_url'); ?>" name="<?php echo $this->get_field_name('static_url'); ?>" type="text" value="<?php echo $static_url; ?>" /></label>
        </p>
        <p><label for="<?php echo $this->get_field_id('max_results'); ?>"><?php _e('Max Results:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('max_results'); ?>" name="<?php echo $this->get_field_name('max_results'); ?>" type="text" value="<?php echo $max_results; ?>" /></label></p>
        <p><label for="<?php echo $this->get_field_id('tz_offset'); ?>"><?php _e('Timezone Offset:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('tz_offset'); ?>" name="<?php echo $this->get_field_name('tz_offset'); ?>" type="text" value="<?php echo $tz_offset; ?>" /></label></p>
        <?php 
    }

/**
 * @function get_title
 * @brief Generates the 'h2' header for the title of the calendar widget
 * @param $feed_title
 *   $feed_title is the text displayed to the user as the title of the widget
 * @param $gcal_sidebar_id
 *   $gcal_sidebar_id is the unique ID of the calendar. This is used to build proper
 *   URLs to link to depending on the value of $feed_title_url_option
 * @param $feed_title_url_option
 *   $feed_title_url_option specifies what the title should link to,
 *   0 is no link, 1 is ical, 2 is HTML, and 3 is a specified link
 *
 * gcal_sidebar_display feed does most of the heavy lifting. It is
 * responsible for downloading the feed, parsing it, and 
 * generating the proper HTML. 
 *
 */
     function get_title($feed_title, $gcal_sidebar_id, $feed_title_url_option) {
             switch($feed_title_url_option) {
	     case 0:
		     $title = $feed_title;
		     break;
	     case 1:
		     $feed_title_url = "http://www.google.com/calendar/ical/" . $gcal_sidebar_id . "/public/basic.ics";
        	     $title = "<a href='" . $feed_title_url . "'>" . $feed_title . "</a>";
		     break;
	     case 2:
		     $feed_title_url = "http://www.google.com/calendar/embed?src=" . $gcal_sidebar_id;
        	     $title = "<a href='" . $feed_title_url . "'>" . $feed_title . "</a>";
		     break;
	     case 3:
		     $feed_title_url = get_option('gcal_sidebar_title_url');
        	     $title = "<a href='" . $feed_title_url . "'>" . $feed_title . "</a>";
		     break;
	     }
             return $title;
     } 

/**
 * @function display_feed
 * @brief Downloads the Google Calendar and converts to HTML
 * @param $feed_id 
 *   $feed_id is the unique ID for a gcal calendar. It uses this to build 
 *   proper URLs for the HTML / ical / XML feeds.
 *
 * gcal_sidebar_display feed does most of the heavy lifting. It is
 * responsible for downloading the feed, parsing it, and 
 * generating the proper HTML. 
 *
 */
    function display_feed( $feed_id, $instance) {
        $max_results = (( $instance['max_results'] == '' ) ? '4' : $instance['max_results']);

        $feed_url = "http://www.google.com/calendar/feeds/" . $feed_id . "/public/full?orderby=starttime&sortorder=ascending&futureevents=true&singleevents=true&max-results=" . $max_results;

        $xmlstr = wp_remote_fopen($feed_url);
        $static_url = get_option('gcal_sidebar_static_url_option');

        $xml = new SimpleXMLElement($xmlstr);

        echo '<ul id="events">';
        foreach($xml->entry as $entry) {
            echo '<li class="event">';

            $gd = $entry->children('http://schemas.google.com/g/2005');
    
            if ( $static_url ) {
                $event_link =  get_option('gcal_sidebar_static_url');
            }
            else {
                $event_link = $entry->link->attributes()->href;
            }
    
                    if(isset($entry->content))
			    $content = $entry->content;
                    else
			    $content = "";
                    if(isset($gd->where))
			    $where = $gd->where->attributes()->valueString;
                    else
			    $where = "";
    
		    $tooltip = $where . " - " . $content;
		    echo '<a title="' . $tooltip . '" href="' . $event_link  . '">' . $entry->title . "</a>\n";
    
		    if (($offset = get_option('gcal_sidebar_timezone_offset')) == '')
		       $offset = 7200;
    
		    $start = date("l, F j \\f\\r\o\m g:ia", strtotime($gd->when->attributes()->startTime) + $offset);
		    $end = date("g:ia", strtotime($gd->when->attributes()->endTime) + $offset);
    
		    echo "<p class='event_time'>$start to $end</p></li>";
	    }
	    echo '</ul>';
    }
} // class GcalSidebar


add_action('widgets_init', create_function('', 'return register_widget("GcalSidebar");'));

?>
